#!/usr/bin/env python

# based on multi_timed_run.py by jesse ruderman, which was
# based on timed_run.py by coop and bob clary

# Usage: nt.py timeout firefoxpath
# (timeout is in seconds, firefoxpath should be absolute)


exitOSError   = 66
exitInterrupt = 99

(CRASHED, TIMED_OUT, NORMAL, ABNORMAL, NONE) = range(5)


import os, signal, sys, time

pid = None
URLs = []


def alarm_handler(signum, frame):
    global pid
    # print "TIMED OUT!"
    try:
        os.kill(pid, signal.SIGKILL)
    except:
        pass

def forkexec(command, args, stdoutfn):
    try:
        newpid = os.fork()
        if newpid == 0:  # Child

            # redirect stdout (just for the child!).
            # i hope i'm not supposed to close() this file somehow...
            so = file(stdoutfn, 'w')
            os.dup2(so.fileno(), sys.stdout.fileno())

            # redirect stderr too (to a separate file, for now)
            so = file("err-" + stdoutfn, 'w')
            os.dup2(so.fileno(), sys.stderr.fileno())
            
            # transfer control of the child to the target program
            os.execvp(command, args)
             
        else:  # Parent
            return newpid
    except OSError, e:
        print "ERROR: %s %s failed: %d (%s)" % (command, args, e.errno, e.strerror)
        sys.exit(exitOSError)

def one_timed_run(logfilename, URL):
    global pid
    global CRASHED, TIMED_OUT, NORMAL, ABNORMAL, NONE

    signal.signal(signal.SIGALRM, alarm_handler)
    signal.alarm(int(sys.argv[1]))

    starttime = time.time()
    
    prog = sys.argv[2]
    args = [prog, URL]

    msg = ''

    try:
      pid = forkexec(prog, args, logfilename)

      try: 
        status = os.waitpid(pid, 0)[1]
      except OSError:
        msg = 'TIMED OUT' # assume this means it timed out
        sta = TIMED_OUT
      
      signal.alarm(0) # Cancel the alarm

    except KeyboardInterrupt:
      print "Bye!"
      try:
        os.kill(pid, signal.SIGKILL)
      except OSError, e:
        print "Unable to kill it: %s" % e
      sys.exit(exitInterrupt)

    stoptime = time.time()
    elapsedtime = stoptime - starttime
    
    if (msg == 'TIMED OUT'):
        pass
    elif os.WIFEXITED(status):
        rc = os.WEXITSTATUS(status)
        if rc == 0:
            msg = 'NORMAL'
            sta = NORMAL
        else:
            # the original timed_run.py considered rc >= 3 to be a crash. why?
            msg = 'ABNORMAL ' + str(rc)
            sta = ABNORMAL
    elif os.WIFSIGNALED(status):
        print time.asctime()
        msg = 'CRASHED signal %d' % os.WTERMSIG(status)
        sta = CRASHED
    else:
        msg = 'NONE'
        sta = NONE

    print "EXIT STATUS: %s (%f seconds) (log file %s)" % (msg, elapsedtime, logfilename)

    return sta

def many_timed_runs():
    global CRASHED, TIMED_OUT, NORMAL, ABNORMAL, NONE
    global URLs
    
    iteration = 0
    while True:
        iteration += 1
        URL = "https://bugzilla.mozilla.org/attachment.cgi?id=262566"
        logfilename = "w%d" % iteration
        sta = one_timed_run(logfilename, URL)
             
many_timed_runs()